# handle the i18n: namespace

from pax.paxtransform import AttributeHandler, StopTransform
from pax import XML
import os, glob
import zLOG, sys
from GlobalTranslationService import GlobalTranslationService
from GettextMessageCatalog import GettextMessageCatalog
from utils import *
from types import TupleType
from i18n_compiler import i18n_ns

default_domain = 'default'

# default (gettext) translation service
translation_service = GlobalTranslationService('default')
basepath = os.path.join(INSTANCE_HOME, 'i18n')
if os.path.isdir(basepath):
    for name in glob.glob(os.path.join(basepath, '*.mo')):
        try:
            translation_service.addCatalog(GettextMessageCatalog(name))
        except ValueError:
            zLOG.LOG('AltPTi18n', zLOG.PROBLEM, 'Message Catalog has no metadata',
                     name, sys.exc_info())
        except:
            zLOG.LOG('AltPTi18n', zLOG.PROBLEM, 'Message Catalog has errors',
                     name, sys.exc_info())


class I18nHandler(object):
    def __init__(self):
        self.name = 'i18n'
        self.ns = i18n_ns

    def initialize(self, context):
        context.i18n_info_stack = []
        context.i18n_domain_stack = [None]
        context.i18n_name_stack = [{}]
        # here we can hook to placeful translation services if we later wish to
        context.translate = translation_service.translate
        self.ns_map = getattr(context, 'root_xmlns_map', None)

    def __call__(self, element, context):
        context.i18n_info_stack.append(element.attributes[i18n_ns])
        domain = getattr(element, 'i18n_domain', context.i18n_domain_stack[-1])
        context.i18n_domain_stack.append(domain)
        context.i18n_name_stack.append({})
        return element

    def postprocess(self, element, context):
        # this one does the real work
        # this is done on postprocess and not in a transform to better interact
        # with tal:replace, tal:omit-tag, etc
        info = context.i18n_info_stack.pop()
        domain = context.i18n_domain_stack.pop() or default_domain
        names = context.i18n_name_stack.pop()
        request = context.global_vars['request']
        if not element or type(element) is TupleType:
            # oops - we have been removed, probably by tal:condition: bail out
            return element
        try:
            del element.attributes[i18n_ns]
        except:
            pass
        #FIXME: domain is TALES?
        if info.has_key('translate') or info.has_key('attributes'):
            if info.has_key('target'):
                target = context.engine.evaluate(info['target'])
            else:
                target = None
        if info.has_key('translate'):
            msgid = info['translate']
            if hasattr(element, 'children'):
                default = XML(element.children, self.ns_map)
            else:
                default = XML(element, self.ns_map)
            default = ' '.join(default.split())
            if msgid == '':
                msgid = default
                default = None
            if info.has_key('data'):
                data = context.engine.evaluate(info['data'])
            elif names:
                data = names
            else:
                data = None
            result = Literal(context.translate(domain, msgid, data,
                                               request, target, default))
            if hasattr(element, 'children'):
                element.children = [result]
            else:
                element = result
        if info.has_key('attributes') and hasattr(element, 'attributes'):
            attrs = element.attributes.setdefault(element.ns, {})
            # FIXME: the spec doesn't say what is the separator!!!
            for name in info['attributes'].split():
                attrs[name] = context.translate(domain, attrs[name], None,
                                                request, target)
        if info.has_key('name'):
            name = info['name']
            element = context.tr_engine.postprocess(element, context)
            context.i18n_name_stack[-1][name] = element
            element = u'${%s}' % name
        return element

i18n_handler = I18nHandler()
